@@ -4,7 +4,7 @@
 
 """``jeni`` injects annotated dependencies"""
 
-__version__ = '0.3.1'
+__version__ = '0.3.3'
 
 import abc
 import collections
@@ -20,6 +20,7 @@ MAYBE = 'maybe'
 PARTIAL = 'partial'
 
 
+
 class UnsetError(LookupError):
     """Note is not able to be provided, as it is currently unset."""
     def __init__(self, *a, **kw):
@@ -44,7 +45,18 @@ class Provider(object):
         """
 
     def close(self):
-        """By default, does nothing. Close objects as needed in subclass."""
+        """By default, does nothing. Close objects as needed in subclass.
+
+        Provider close methods should not intentionally raise errors.
+        Specifically, if a dependency has transactions, the transaction should
+        be committed or rolled back before close is called, and not left as an
+        operation to be called during the close phase.
+
+        Provider close methods must not take an argument; an injector cannot
+        apply provided values on a close method since some providers may have
+        already been closed. If an injected value is needed for the close
+        method, annotate ``__init__`` and access the value via `self`.
+        """
 
 
 class GeneratorProvider(Provider):
@@ -228,9 +240,9 @@ class Annotator(object):
     def maybe(self, note):
         """Wrap a keyword note to record that its resolution is optional.
 
-        Normally all annotations require fulfilled dependencies, but if a keyword
-        argument is annotated as `maybe`, then an injector does not pass unset
-        dependencies on apply::
+        Normally all annotations require fulfilled dependencies, but if a
+        keyword argument is annotated as `maybe`, then on apply, an injector
+        does not attempt to pass dependencies which are unset or not provided::
 
             from jeni import annotate
 
@@ -437,11 +449,6 @@ class Injector(object):
     def close(self):
         """Close injector & injected Provider instances, including generators.
 
-        Provider close methods should not intentionally raise errors.
-        Specifically, if a dependency has transactions, the transaction should
-        be committed or rolled back before close is called, and not left as an
-        operation to be called during the close phase.
-
         Providers are closed in the reverse order in which they were opened,
         and each provider is only closed once. Providers are only closed if
         they have successfully provided a dependency via get.
@@ -452,6 +459,7 @@ class Injector(object):
             if basenote not in self.instances:
                 # Provider is not an instance; no close implementation.
                 continue
+            # Note: Unable to apply injector on close method.
             self.instances[basenote].close()
         self.closed = True
 
@@ -470,7 +478,7 @@ class Injector(object):
             if isinstance(note, tuple) and len(note) == 2 and note[0] == MAYBE:
                 try:
                     kwargs[arg] = self.get(note[1])
-                except UnsetError:
+                except LookupError:
                     continue
             else:
                 kwargs[arg] = self.get(note)
@@ -503,7 +511,13 @@ class Injector(object):
         if basenote in self.instances:
             provider_or_fn = self.instances[basenote]
         elif inspect.isclass(provider_or_fn):
-            provider_or_fn = provider_or_fn()
+            # Inject class __init__, if annotated.
+            cls = provider_or_fn
+            if hasattr(cls, '__init__') and self.has_annotations(cls.__init__):
+                args, kwargs = self.prepare_callable(cls.__init__)
+                provider_or_fn = provider_or_fn(*args, **kwargs)
+            else:
+                provider_or_fn = provider_or_fn()
             self.instances[basenote] = provider_or_fn
         elif inspect.isgeneratorfunction(provider_or_fn):
             provider_or_fn, value = self.init_generator(provider_or_fn)
@@ -526,8 +540,12 @@ class Injector(object):
         except UnsetError:
             # Use sys.exc_info to support both Python 2 and Python 3.
             exc_type, exc_value, tb = sys.exc_info()
-            exc_value.note = note
-            six.reraise(exc_type, exc_value, tb)
+            exc_msg = str(exc_value)
+            if exc_msg:
+                msg = '{}: {!r}'.format(exc_msg, note)
+            else:
+                msg = repr(note)
+            six.reraise(exc_type, exc_type(msg, note=note), tb)
 
     @classmethod
     def register(cls, note, provider):
